import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/*
 * Created on 18 mai 2003
 *
 * (C) S. Rosmorduc
 * This file is free software under the LGLP
 * 
 */

/**
 * @author rosmord
 */
public class DialogueClient extends UnicastRemoteObject implements RemoteObserverInterface {

	String host;
	int lastMessage;
	int clientID;
	DialogueInterface server;

	public DialogueClient(String host) throws RemoteException {
		this.host= host;
		lastMessage= 0;
	}

	/**
	 * Appele quand l'objet est dtruit. C'est trs important car sinon il reste li dans
	 * le serveur. 
	 * @see java.lang.Object#finalize()
	 */
	public void finalize() {
		System.out.println("calling finalize");
		deconnect();
	}
	
	private void deconnect() {
		try {
				server.removeObserver(this);
				server=null;
				Naming.unbind(DialogueNamingPolicy.getClientName(host, clientID));
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (NotBoundException e) {
				e.printStackTrace();
			}
	}
	
	public void run()
		throws MalformedURLException, RemoteException, NotBoundException {
		// Create and install a security manager
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}
		// Find the server
		server=
			(DialogueInterface) Naming.lookup(
				DialogueNamingPolicy.getServerName(host));
		// get a client id from the server
		clientID= server.getNextClientID();
		// Publish self as Observer
		Naming.rebind(DialogueNamingPolicy.getClientName(host, clientID), this);
		// Register as Observer with the client.
		// What should be published ? The object itself, or the reference ?
		server.addObserver(this);
		// we are done.
		doClient();
		deconnect();
		System.out.println("bye.");
		System.exit(0);
	}
		
	public void update() throws RemoteException {
		int j= server.getSize();
		for (int i= lastMessage; i < j; i++) {
			System.out.println(server.getLine(i));
		}
		lastMessage= j;
	}

	private void doClient() {
		String line;
		BufferedReader r= new BufferedReader(new InputStreamReader(System.in));
		try {
			while ((line= r.readLine()) != null) {
				server.addLine(line);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println("fin");
	}

	public static void main(String[] args)
		throws MalformedURLException, RemoteException, NotBoundException {
	    if (args.length != 1) {
		System.err.println("Erreur : donnez le nom du serveur de nommage (celui qui fait tourner rmiregistry)");
		System.exit(1);
	    }
	    DialogueClient c= new DialogueClient(args[0]);
	    c.run();
	}
}
